@@ -214,6 +214,7 @@ KDE_NO_EXPORT bool MediaInfo::wget (const String &str) {
     }
 
     job = asyncGet (this, url);
+    job->start ();
     return false;
 }
 
@@ -366,7 +367,7 @@ KDE_NO_EXPORT void MediaInfo::ready () {
     if (ActorAgent::Data == type)
         node->message (MsgMediaReady);
     else
-        node->document()->post (node, new Posting (node, MsgMediaReady));
+        node->document()->post (new Posting (node, node, MsgMediaReady));
 }
 
 static bool validDataFormat (ActorAgent::ActorType type, const ByteArray &ba) {
@@ -646,21 +647,21 @@ unsigned ImageActor::render (cairo_surface_t *similar, ISize sz) {
 KDE_NO_EXPORT void ImageActor::updateRender () {
     update_render = true;
     if (m_node)
-        m_node->document()->post(m_node, new Posting (m_node, MsgMediaUpdated));
+        m_node->document()->post(new Posting (m_node, m_node, MsgMediaUpdated));
 }
 
-KDE_NO_EXPORT void ImageActor::sizes (Single &width, Single &height) {
+KDE_NO_EXPORT void ImageActor::sizes (SSize &size) {
     if (svg_renderer) {
         RsvgDimensionData dim;
         rsvg_handle_get_dimensions (svg_renderer, &dim);
-        width = dim.width;
-        height = dim.height;
+        size.width = dim.width;
+        size.height = dim.height;
     } else if (cached_img) {
-        width = cached_img->width;
-        height = cached_img->height;
+        size.width = cached_img->width;
+        size.height = cached_img->height;
     } else {
-        width = 0;
-        height = 0;
+        size.width = 0;
+        size.height = 0;
     }
 }
 
@@ -684,9 +685,9 @@ bool ImageActor::movieTimer () {
                 img_movie_timer = g_timeout_add (to, movieUpdateTimeout, this);
         }
         if (m_node)
-            m_node->document()->post (m_node, new Posting (m_node, MsgMediaUpdated));
+            m_node->document()->post (new Posting (m_node, m_node, MsgMediaUpdated));
     } else if (m_node) {
-        m_node->document()->post (m_node, new Posting(m_node, MsgMediaFinished));
+        m_node->document()->post (new Posting(m_node, m_node, MsgMediaFinished));
     }
     return ret;
 }
