@@ -111,10 +111,13 @@ Recent::Recent (KMPlayer::NodePtr & doc, Application * a, const String &url)
 KDE_NO_EXPORT void Recent::closed () {
     if (src.isEmpty ())
         src = getAttribute (KMPlayer::StringPool::attr_url);
+    if (title.isEmpty ())
+        title = getAttribute (KMPlayer::StringPool::attr_title);
 }
 
 KDE_NO_EXPORT void Recent::activate () {
     Document *doc = new SourceDocument (CONTROL (app), src);
+    doc->setCaption (title);
     String pos = getAttribute ("clipBegin");
     if (!pos.isEmpty ())
         doc->clip_begin = parseTimeString (pos);
@@ -438,6 +441,8 @@ String Generator::genReadString (Control *ctr, Node *n) {
         case id_node_text:
              str += c->nodeValue ().stripWhiteSpace ();
         }
+    if (!static_cast <Element *>(n)->getAttribute ("encoding").isEmpty ())
+        str = URL::encode_string (str);
     if (need_quote) {
         char *quoted = g_shell_quote ((const char *) str);
         str = quoted;
