@@ -112,8 +112,8 @@ KMPLAYER_NO_EXPORT void RSS::Item::activate () {
     if (!enclosure && !s.isEmpty ()) {
         setState (state_activated);
         begin ();
-        timer = document ()->post (this,
-                new TimerPosting (5000+s.length()*200));
+        timer = document ()->post (
+                new TimerPosting (this, 5000+s.length()*200));
         return;
     }
     Mrl::activate ();
