@@ -67,16 +67,22 @@ Node * KMPlayer::fromXMLDocumentTag (NodePtr & d, const String & tag) {
 
 //-----------------------------------------------------------------------------
 
+static const Char char_lt ('<');
+static const Char char_gt ('>');
+static const Char char_quot ('"');
+static const Char char_amp ('&');
+static const Char char_nl ('\n');
+
 TextStream &KMPlayer::operator << (TextStream &out, const XMLStringlet &txt) {
     int len = int (txt.str.length ());
     for (int i = 0; i < len; ++i) {
-        if (txt.str [i] == Char ('<')) {
+        if (txt.str [i] == char_lt) {
             out <<  "&lt;";
-        } else if (txt.str [i] == Char ('>')) {
+        } else if (txt.str [i] == char_gt) {
             out <<  "&gt;";
-        } else if (txt.str [i] == Char ('"')) {
+        } else if (txt.str [i] == char_quot) {
             out <<  "&quot;";
-        } else if (txt.str [i] == Char ('&')) {
+        } else if (txt.str [i] == char_amp) {
             out <<  "&amp;";
         } else
             out << txt.str [i];
@@ -105,14 +111,6 @@ KDE_NO_EXPORT void Connection::disconnect () {
 
 //-----------------------------------------------------------------------------
 
-KDE_NO_CDTOR_EXPORT TimerPosting::TimerPosting (int ms, unsigned eid)
- : Posting (NULL, MsgEventTimer),
-   event_id (eid),
-   milli_sec (ms),
-   interval (false) {}
-
-//-----------------------------------------------------------------------------
-
 KDE_NO_CDTOR_EXPORT Node::Node (NodePtr & d, short _id)
  : m_doc (d), state (state_init), id (_id),
    auxiliary_node (false), editable (true), open (false) {}
@@ -196,7 +194,7 @@ void Node::finish () {
     if (active ()) {
         setState (state_finished);
         if (m_parent)
-            document ()->post (m_parent, new Posting (this, MsgChildFinished));
+            document ()->post (new Posting (this, m_parent, MsgChildFinished));
         else
             deactivate (); // document deactivates itself on finish
     } else
@@ -215,7 +213,7 @@ void Node::deactivate () {
             break; // remaining not yet activated
     }
     if (need_finish && m_parent && m_parent->active ())
-        document ()->post (m_parent, new Posting (this, MsgChildFinished));
+        document ()->post (new Posting (this, m_parent, MsgChildFinished));
 }
 
 void Node::reset () {
@@ -324,7 +322,7 @@ void Node::normalize () {
 static void getInnerText (const Node *p, TextStream & out) {
     for (Node *e = p->firstChild (); e; e = e->nextSibling ()) {
         if (e->id == id_node_text || e->id == id_node_cdata)
-            out << e->nodeValue () << Char ('\n');
+            out << e->nodeValue () << char_nl;
         else
             getInnerText (e, out);
     }
@@ -339,24 +337,24 @@ String Node::innerText () const {
 static void getOuterXML (const Node *p, TextStream & out, int depth) {
     if (!p->isElementNode ()) { // #text or #cdata
         if (p->id == id_node_cdata)
-            out << "<![CDATA[" << p->nodeValue () << "]]>" << Char ('\n');
+            out << "<![CDATA[" << p->nodeValue () << "]]>" << char_nl;
         else
-            out << XMLStringlet (p->nodeValue ()) << Char ('\n');
+            out << XMLStringlet (p->nodeValue ()) << char_nl;
     } else {
         const Element *e = static_cast <const Element *> (p);
         String indent (String ().fill (Char (' '), depth));
-        out << indent << Char ('<') << XMLStringlet (e->nodeName ());
+        out << indent << char_lt << XMLStringlet (e->nodeName ());
         for (Attribute *a = e->attributes()->first(); a; a = a->nextSibling())
             out << " " << XMLStringlet (a->name ().toString ()) <<
                 "=\"" << XMLStringlet (a->value ()) << "\"";
         if (e->hasChildNodes ()) {
-            out << Char ('>') << Char ('\n');
+            out << char_gt << char_nl;
             for (Node *c = e->firstChild (); c; c = c->nextSibling ())
                 getOuterXML (c, out, depth + 1);
             out << indent << String("</") << XMLStringlet (e->nodeName()) <<
-                Char ('>') << Char ('\n');
+                char_gt << char_nl;
         } else
-            out << String ("/>") << Char ('\n');
+            out << String ("/>") << char_nl;
     }
 }
 
@@ -806,11 +804,12 @@ void Mrl::parseParam (const TrieString & para, const String & val) {
 
 //-----------------------------------------------------------------------------
 
-EventData::EventData (Node *t, Posting *e, EventData *n)
- : target (t), event (e), next (n) {}
+CommandData::CommandData (Command *e, CommandData *n)
+ : event (e), next (n) {}
 
-EventData::~EventData () {
-    delete event;
+CommandData::~CommandData () {
+    if (event)
+        event->removed ();
 }
 //-----------------------------------------------------------------------------
 
@@ -910,7 +909,7 @@ void Document::reset () {
     }
     if (event_queue) {
         while (event_queue) {
-            EventData *ed = event_queue;
+            CommandData *ed = event_queue;
             event_queue = ed->next;
             delete ed;
         }
@@ -929,6 +928,50 @@ static inline void addTime (struct timeval & tv, int ms) {
     tv.tv_usec = (tv.tv_usec + ms*1000) % 1000000;
 }
 
+//-----------------------------------------------------------------------------
+
+void Posting::apply (struct timeval &) {
+    if (target)
+        target->message (message, this);
+    delete this;
+}
+
+bool Posting::postponable () const {
+    return message == MsgEventTimer ||
+        message == MsgEventStarted ||
+        message == MsgEventStopped;
+}
+
+int Posting::delay () const {
+    return 0;
+}
+
+void Posting::removed () {
+    delete this;
+}
+
+KDE_NO_CDTOR_EXPORT TimerPosting::TimerPosting (Node *t, int ms, unsigned eid)
+ : Posting (NULL, t, MsgEventTimer),
+   event_id (eid),
+   milli_sec (ms),
+   interval (false) {}
+
+void TimerPosting::apply (struct timeval &timeout) {
+    if (target)
+        target->message (message, this);
+    if (target && interval) {
+        interval = false; // reset interval
+        addTime (timeout, milli_sec);
+        target->document ()->insertPosting (this, timeout);
+    } else {
+        delete this;
+    }
+}
+
+int TimerPosting::delay () const {
+    return milli_sec;
+}
+
 KDE_NO_CDTOR_EXPORT UpdateEvent::UpdateEvent (Document *doc, unsigned int skip)
  : skipped_time (skip) {
     struct timeval tv;
@@ -959,26 +1002,20 @@ void Document::timeOfDay (struct timeval & tv) {
     }
 }
 
-static bool postponedSensible (MessageType msg) {
-    return msg == MsgEventTimer ||
-        msg == MsgEventStarted ||
-        msg == MsgEventStopped;
-}
-
-void Document::insertPosting (Node *n, Posting *e, const struct timeval &tv) {
+void Document::insertPosting (Command *e, const struct timeval &tv) {
     if (!notify_listener)
         return;
-    bool postponed_sensible = postponedSensible (e->message);
-    EventData *prev = NULL;
-    EventData *ed = event_queue;
+    bool postponed_sensible = e->postponable ();
+    CommandData *prev = NULL;
+    CommandData *ed = event_queue;
     for (; ed; ed = ed->next) {
         int diff = diffTime (ed->timeout, tv);
-        bool psens = postponedSensible (ed->event->message);
+        bool psens = ed->event->postponable ();
         if ((diff > 0 && postponed_sensible == psens) || (!postponed_sensible && psens))
             break;
         prev = ed;
     }
-    ed = new EventData (n, e, ed);
+    ed = new CommandData (e, ed);
     ed->timeout = tv;
     if (prev)
         prev->next = ed;
@@ -991,7 +1028,7 @@ void Document::setNextTimeout (const struct timeval &now) {
     if (!cur_event) {              // if we're not processing events
         int timeout = 0x7FFFFFFF;
         if (event_queue && active () &&
-                (!postpone_ref || !postponedSensible (event_queue->event->message)))
+                (!postpone_ref || !event_queue->event->postponable ()))
             timeout = diffTime (event_queue->timeout, now);
         timeout = 0x7FFFFFFF != timeout
             ? (timeout > 0 ? timeout : 0)
@@ -1016,10 +1053,8 @@ void Document::updateTimeout () {
     }
 }
 
-Posting *Document::post (Node *n, Posting *e) {
-    int ms = e->message == MsgEventTimer
-        ? static_cast<TimerPosting *>(e)->milli_sec
-        : 0;
+Command *Document::post (Command *e) {
+    int ms = e->delay ();
     struct timeval now, tv;
     if (cur_event)
         now = cur_event->timeout;
@@ -1027,15 +1062,15 @@ Posting *Document::post (Node *n, Posting *e) {
         timeOfDay (now);
     tv = now;
     addTime (tv, ms);
-    insertPosting (n, e, tv);
+    insertPosting (e, tv);
     if (postpone_ref || event_queue->event == e)
         setNextTimeout (now);
     return e;
 }
 
-static EventData *findPosting (EventData *queue, EventData **prev, const Posting *e) {
+static CommandData *findPosting (CommandData *queue, CommandData **prev, const Command *e) {
     *prev = NULL;
-    for (EventData *ed = queue; ed; ed = ed->next) {
+    for (CommandData *ed = queue; ed; ed = ed->next) {
         if (e == ed->event)
             return ed;
         *prev = ed;
@@ -1043,14 +1078,14 @@ static EventData *findPosting (EventData *queue, EventData **prev, const Posting
     return NULL;
 }
 
-void Document::cancelPosting (Posting *e) {
+void Document::cancelPosting (Command *e) {
     if (cur_event && cur_event->event == e) {
-        delete cur_event->event;
+        cur_event->event->removed ();
         cur_event->event = NULL;
     } else {
-        EventData *prev;
-        EventData **queue = &event_queue;
-        EventData *ed = findPosting (event_queue, &prev, e);
+        CommandData *prev;
+        CommandData **queue = &event_queue;
+        CommandData *ed = findPosting (event_queue, &prev, e);
         if (!ed) {
             ed = findPosting (paused_queue, &prev, e);
             queue = &paused_queue;
@@ -1069,19 +1104,19 @@ void Document::cancelPosting (Posting *e) {
             }
             delete ed;
         } else {
-            errorLog () << "Posting not found" << endl;
+            errorLog () << "Command not found" << endl;
         }
     }
 }
 
-void Document::pausePosting (Posting *e) {
+void Document::pausePosting (Command *e) {
     if (cur_event && cur_event->event == e) {
-        paused_queue = new EventData (cur_event->target, cur_event->event, paused_queue);
+        paused_queue = new CommandData (cur_event->event, paused_queue);
         paused_queue->timeout = cur_event->timeout;
         cur_event->event = NULL;
     } else {
-        EventData *prev;
-        EventData *ed = findPosting (event_queue, &prev, e);
+        CommandData *prev;
+        CommandData *ed = findPosting (event_queue, &prev, e);
         if (ed) {
             if (prev)
                 prev->next = ed->next;
@@ -1095,16 +1130,16 @@ void Document::pausePosting (Posting *e) {
     }
 }
 
-void Document::unpausePosting (Posting *e, int ms) {
-    EventData *prev;
-    EventData *ed = findPosting (paused_queue, &prev, e);
+void Document::unpausePosting (Command *e, int ms) {
+    CommandData *prev;
+    CommandData *ed = findPosting (paused_queue, &prev, e);
     if (ed) {
         if (prev)
             prev->next = ed->next;
         else
             paused_queue = ed->next;
         addTime (ed->timeout, ms);
-        insertPosting (ed->target, ed->event, ed->timeout);
+        insertPosting (ed->event, ed->timeout);
         ed->event = NULL;
         delete ed;
     } else {
@@ -1122,32 +1157,17 @@ void Document::timer () {
 
         // handle max 100 timeouts with timeout set to now
         for (int i = 0; i < 100 && active (); ++i) {
-            if (postpone_ref && postponedSensible (cur_event->event->message))
+            if (postpone_ref && cur_event->event->postponable ())
                 break;
             // remove from queue
             event_queue = cur_event->next;
 
-            if (!cur_event->target) {
-                // some part of document has gone and didn't remove timer
-                errorLog () << "spurious timer" << endl;
-            } else {
-                EventData *ed = cur_event;
-                cur_event->target->message (cur_event->event->message, cur_event->event);
-                if (!guard) {
-                    delete ed;
-                    return;
-                }
-                if (cur_event->event && cur_event->event->message == MsgEventTimer) {
-                    TimerPosting *te = static_cast <TimerPosting *> (cur_event->event);
-                    if (te->interval) {
-                        te->interval = false; // reset interval
-                        addTime (cur_event->timeout, te->milli_sec);
-                        insertPosting (cur_event->target,
-                                cur_event->event,
-                                cur_event->timeout);
-                        cur_event->event = NULL;
-                    }
-                }
+            CommandData *ed = cur_event;
+            cur_event->event->apply (cur_event->timeout);
+            ed->event = NULL;
+            if (!guard) {
+                delete ed;
+                return;
             }
             delete cur_event;
             cur_event = event_queue;
@@ -1185,8 +1205,8 @@ void Document::proceed (const struct timeval &postponed_time) {
     timeOfDay (now);
     int diff = diffTime (now, postponed_time);
     if (event_queue) {
-        for (EventData *ed = event_queue; ed; ed = ed->next)
-            if (ed->event && postponedSensible (ed->event->message))
+        for (CommandData *ed = event_queue; ed; ed = ed->next)
+            if (ed->event && ed->event->postponable ())
                 addTime (ed->timeout, diff);
         setNextTimeout (now);
     }
@@ -1477,7 +1497,7 @@ void KMPlayer::readXML (NodePtr root, TextStream & in, const String & firstline,
     XML_SetCharacterDataHandler (parser, characterData);
     XML_SetCdataSectionHandler (parser, cdataStart, cdataEnd);
     if (!firstline.isEmpty ()) {
-        String str (firstline + Char ('\n'));
+        String str (firstline + char_nl);
         QCString buf = str.utf8 ();
         ok = XML_Parse(parser, buf, strlen (buf), false) != XML_STATUS_ERROR;
         if (!ok)
@@ -1500,7 +1520,20 @@ void KMPlayer::readXML (NodePtr root, TextStream & in, const String & firstline,
 namespace {
 
 class SimpleSAXParser {
+    enum Token { tok_empty, tok_text, tok_white_space, tok_angle_open,
+        tok_equal, tok_double_quote, tok_single_quote, tok_angle_close,
+        tok_slash, tok_exclamation, tok_amp, tok_hash, tok_semi_colon,
+        tok_question_mark, tok_cdata_start };
 public:
+    struct TokenInfo {
+        TokenInfo () : token (tok_empty) {}
+        void *operator new (size_t);
+        void operator delete (void *);
+        Token token;
+        String string;
+        SharedPtr <TokenInfo> next;
+    };
+    typedef SharedPtr <TokenInfo> TokenInfoPtr;
     SimpleSAXParser (DocumentBuilder & b) : builder (b), position (0), m_attributes (new AttributeList), equal_seen (false), in_dbl_quote (false), in_sngl_quote (false), have_error (false), no_entitity_look_ahead (false), have_next_char (false) {}
     virtual ~SimpleSAXParser () {};
     bool parse (TextStream & d);
@@ -1509,20 +1542,9 @@ private:
     DocumentBuilder & builder;
     int position;
     Char next_char;
-    enum Token { tok_empty, tok_text, tok_white_space, tok_angle_open,
-        tok_equal, tok_double_quote, tok_single_quote, tok_angle_close,
-        tok_slash, tok_exclamation, tok_amp, tok_hash, tok_semi_colon,
-        tok_question_mark, tok_cdata_start };
     enum State {
         InTag, InStartTag, InPITag, InDTDTag, InEndTag, InAttributes, InContent, InCDATA, InComment
     };
-    struct TokenInfo {
-        TokenInfo () : token (tok_empty) {}
-        Token token;
-        String string;
-        SharedPtr <TokenInfo> next;
-    };
-    typedef SharedPtr <TokenInfo> TokenInfoPtr;
     struct StateInfo {
         StateInfo (State s, SharedPtr <StateInfo> n) : state (s), next (n) {}
         State state;
@@ -1555,14 +1577,82 @@ private:
     void push_attribute ();
 };
 
+static struct TokenPool {
+    void **used;
+    void **unused;
+    int used_count;
+    int unused_count;
+    int used_size;
+    int unused_size;
+    void *alloc ();
+    void dealloc (void *p);
+    TokenPool ()
+        : used (NULL), unused (NULL),
+          used_count (0), unused_count (0),
+          used_size (0), unused_size (0) {}
+} token_pool;
+
 } // namespace
 
+inline void *TokenPool::alloc () {
+    if (used_size == used_count) {
+        void **tmp = (void **) malloc (++used_size * sizeof (void *));
+        if (used_count) {
+            memcpy (tmp, used, used_count * sizeof (void *));
+            free (used);
+        }
+        used = tmp;
+    }
+    void *p;
+    if (unused_count)
+        p = unused[--unused_count];
+    else
+        p = malloc (sizeof (SimpleSAXParser::TokenInfo));
+    used[used_count++] = p;
+    return p;
+}
+
+inline void TokenPool::dealloc (void *p) {
+    for (int i = 0; i < used_count; ++i)
+        if (p == used[i]) {
+            if (unused_size == unused_count) {
+                void **tmp = (void **) malloc (++unused_size * sizeof (void*));
+                if (unused_count) {
+                    memcpy (tmp, unused, unused_count * sizeof (void *));
+                    free (unused);
+                }
+                unused = tmp;
+            }
+            unused[unused_count++] = p;
+            if (i < used_count-1)
+                used[i] = used[used_count-1];
+            --used_count;
+            break;
+        }
+}
+
+void *SimpleSAXParser::TokenInfo::operator new (size_t s) {
+    return token_pool.alloc ();
+}
+
+void SimpleSAXParser::TokenInfo::operator delete (void *p) {
+    token_pool.dealloc (p);
+}
+
+static const Char char_semi (';');
+static const Char char_eq ('=');
+static const Char char_squot ('\'');
+static const Char char_qu ('?');
+static const Char char_hash ('#');
+static const Char char_bang ('!');
+static const Char char_slash ('/');
+
 KMPLAYER_EXPORT
 void KMPlayer::readXML (NodePtr root, TextStream & in, const String & firstline, bool set_opener) {
     DocumentBuilder builder (root, set_opener);
     SimpleSAXParser parser (builder);
     if (!firstline.isEmpty ()) {
-        String str (firstline + Char ('\n'));
+        String str (firstline + char_nl);
         TextStream fl_in (str /*, IO_ReadOnly*/);
         parser.parse (fl_in);
     }
@@ -1578,7 +1668,7 @@ void KMPlayer::readXML (NodePtr root, TextStream & in, const String & firstline,
 }
 
 void SimpleSAXParser::push () {
-    if (next_token->string.length ()) {
+    if (!next_token->string.isEmpty ()) {
         prev_token = token;
         token = next_token;
         if (prev_token)
@@ -1599,74 +1689,78 @@ void SimpleSAXParser::push_attribute () {
 bool SimpleSAXParser::nextToken () {
     TokenInfoPtr cur_token = token;
     while (!data->atEnd () && cur_token == token && !(token && token->next)) {
-        if (have_next_char)
+        const gchar *at = NULL;
+        if (have_next_char) {
             have_next_char = false;
-        else
+        } else {
+            at = data->at ();
             *data >> next_char;
+        }
         bool append_char = true;
         if (next_char.isSpace ()) {
             if (next_token->token != tok_white_space)
                 push ();
             next_token->token = tok_white_space;
         } else if (!next_char.isLetterOrNumber ()) {
-            if (next_char == Char ('#')) {
+            if (next_char == char_hash) {
                 //if (next_token->token == tok_empty) { // check last item on stack &
                     push ();
                     next_token->token = tok_hash;
                 //}
-            } else if (next_char == Char ('/')) {
+            } else if (next_char == char_slash) {
                 push ();
                 next_token->token = tok_slash;
-            } else if (next_char == Char ('!')) {
+            } else if (next_char == char_bang) {
                 push ();
                 next_token->token = tok_exclamation;
-            } else if (next_char == Char ('?')) {
+            } else if (next_char == char_qu) {
                 push ();
                 next_token->token = tok_question_mark;
-            } else if (next_char == Char ('<')) {
+            } else if (next_char == char_lt) {
                 push ();
                 next_token->token = tok_angle_open;
-            } else if (next_char == Char ('>')) {
+            } else if (next_char == char_gt) {
                 push ();
                 next_token->token = tok_angle_close;
-            } else if (next_char == Char (';')) {
+            } else if (next_char == char_semi) {
                 push ();
                 next_token->token = tok_semi_colon;
-            } else if (next_char == Char ('=')) {
+            } else if (next_char == char_eq) {
                 push ();
                 next_token->token = tok_equal;
-            } else if (next_char == Char ('"')) {
+            } else if (next_char == char_quot) {
                 push ();
                 next_token->token = tok_double_quote;
-            } else if (next_char == Char ('\'')) {
+            } else if (next_char == char_squot) {
                 push ();
                 next_token->token = tok_single_quote;
-            } else if (next_char == Char ('&')) {
+            } else if (next_char == char_amp) {
                 push ();
                 if (no_entitity_look_ahead) {
                     have_next_char = true;
                     break;
                 }
                 append_char = false;
+                at = NULL;
                 no_entitity_look_ahead = true;
                 TokenInfoPtr tmp = token;
                 TokenInfoPtr prev_tmp = prev_token;
                 if (nextToken () && token->token == tok_text &&
                         nextToken () && token->token == tok_semi_colon) {
                     if (prev_token->string == String ("amp"))
-                        token->string = Char ('&');
+                        token->string = char_amp;
                     else if (prev_token->string == String ("lt"))
-                        token->string = Char ('<');
+                        token->string = char_lt;
                     else if (prev_token->string == String ("gt"))
-                        token->string = Char ('>');
+                        token->string = char_gt;
                     else if (prev_token->string == String ("quot"))
-                        token->string = Char ('"');
+                        token->string = char_quot;
                     else if (prev_token->string == String ("apos"))
-                        token->string = Char ('\'');
+                        token->string = char_squot;
                     else if (prev_token->string == String ("copy"))
                         token->string = Char (169);
                     else
-                        token->string = Char ('?');// TODO lookup more ..
+                        token->string = char_qu;// TODO lookup more ..
                     token->token = tok_text;
                     if (tmp) { // cut out the & xxx ; tokens
                         tmp->next = token;
@@ -1690,7 +1784,7 @@ bool SimpleSAXParser::nextToken () {
                     token = tmp; // restore and insert the lost & token
                     tmp = TokenInfoPtr (new TokenInfo);
                     tmp->token = tok_amp;
-                    tmp->string += Char ('&');
+                    tmp->string += char_amp;
                     tmp->next = token->next;
                     if (token)
                         token->next = tmp;
@@ -1707,8 +1801,19 @@ bool SimpleSAXParser::nextToken () {
             push ();
             next_token->token = tok_text;
         }
-        if (append_char)
+        if (at && !data->atEnd () && tok_text == next_token->token) {
+            const gchar *end;
+            do {
+                end = data->at ();
+                *data >> next_char;
+            } while (!data->atEnd () && next_char.isLetterOrNumber ());
+            String txt;
+            txt.takeUtf8 (g_strndup (at, end - at), end - at);
+            next_token->string += txt;
+            have_next_char = true;
+        } else if (append_char) {
             next_token->string += next_char;
+        }
         if (next_token->token == tok_text &&
                 next_char == Char ('[' ) && next_token->string == "[CDATA[") {
             next_token->token = tok_cdata_start;
@@ -1719,7 +1824,7 @@ bool SimpleSAXParser::nextToken () {
         if (token && token->next) {
             prev_token = token;
             token = token->next;
-        } else if (next_token->string.length ()) {
+        } else if (!next_token->string.isEmpty ()) {
             push (); // last token
         } else
             return false;
@@ -1856,7 +1961,7 @@ bool SimpleSAXParser::readDTD () {
 bool SimpleSAXParser::readCDATA () {
     while (!data->atEnd ()) {
         *data >> next_char;
-        if (next_char == Char ('>') && cdata.endsWith (String ("]]"))) {
+        if (next_char == char_gt && cdata.endsWith (String ("]]"))) {
             cdata.truncate (cdata.length () - 2);
             m_state = m_state->next;
             if (m_state->state == InContent)
@@ -1975,7 +2080,7 @@ bool SimpleSAXParser::parse (TextStream & d) {
                     } else {
                         if (!white_space.isEmpty ()) {
                             if (!in_character_data) {
-                                int pos = white_space.lastIndexOf (Char ('\n'));
+                                int pos = white_space.lastIndexOf (char_nl);
                                 if (pos > -1)
                                     white_space = white_space.mid (pos + 1);
                             }
