@@ -95,15 +95,21 @@ GLibString::GLibString (const Char & ch) {
     gchar buf [8];
     int nr = g_unichar_to_utf8 (ch.unicode (), buf);
     buf [nr] = 0;
-    takeUtf8 (g_strdup (buf));
+    takeUtf8 (g_strdup (buf), nr);
 }
 
-void GLibString::takeUtf8 (char * p) {
+void GLibString::takeUtf8 (char *p, int buf_len) {
     if (!data || data->ptr != p) {
-        SharedChars *tmp = p ? new SharedChars (p) : 0L;
-        if (data)
-            data->unref ();
-        data = tmp;
+        if (data && 1 == data->ref_count) {
+            g_free (data->ptr);
+            data->ptr = p;
+            data->buf_len = buf_len;
+        } else {
+            SharedChars *tmp = p ? new SharedChars (p, buf_len) : 0L;
+            if (data)
+                data->unref ();
+            data = tmp;
+        }
     }
 }
 
@@ -126,7 +132,7 @@ int GLibString::compare (const GLibString & str) const {
 void GLibString::take (char * s) {
     const gchar *end;
     if (g_utf8_validate (s, -1, &end)) {
-        takeUtf8 (s);
+        takeUtf8 (s, end - s);
     } else {
         gsize bytes_read, bytes_written;
         takeUtf8 (g_locale_to_utf8 (s, -1, &bytes_read, &bytes_written, 0L));
@@ -145,24 +151,53 @@ GLibString & GLibString::operator = (const Char & ch) {
     gchar buf [8];
     int nr = g_unichar_to_utf8 (ch.unicode (), buf);
     buf [nr] = 0;
-    takeUtf8 (g_strdup (buf));
+    takeUtf8 (g_strdup (buf), nr);
     return *this;
 }
 
-GLibString & GLibString::operator += (const GLibString & str) {
-    if (!data) {
-        if (str.data)
-            takeUtf8 (g_strdup (str.data->ptr));
-    } else if (str.data)
-        takeUtf8 (g_strconcat (data->ptr, str.data->ptr, 0L));
+static char *strCat (const gchar *p1, int len1, const gchar *p2, int len2) {
+    char *p = (char *) g_malloc (len1 + len2 + 1);
+    memcpy (p, p1, len1);
+    memcpy (p + len1, p2, len2);
+    p [len1 + len2] = 0;
+    return p;
+}
+
+GLibString & GLibString::operator += (const GLibString &str) {
+    if (str.data) {
+        if (!data) {
+            attach (str);
+        } else {
+            if (data->buf_len < 0)
+                data->buf_len = strlen (data->ptr);
+            int slen = str.data->buf_len;
+            if (slen < 0)
+                slen = strlen (str.data->ptr);
+            if (1 == data->ref_count) {
+                data->ptr = (gchar *) g_realloc (data->ptr,
+                                                 data->buf_len + slen + 1);
+                memcpy (data->ptr + data->buf_len, str.data->ptr, slen);
+                data->buf_len += slen;
+                data->ptr [data->buf_len] = 0;
+            } else {
+                takeUtf8(strCat (data->ptr, data->buf_len, str.data->ptr, slen),
+                         data->buf_len + slen);
+            }
+        }
+    }
     return *this;
 }
 
 KMPLAYER_NO_EXPORT GLibString & GLibString::operator += (const char *str) {
-    if (!data)
-        takeUtf8 (g_strdup (str));
-    else
-        takeUtf8 (g_strconcat (data->ptr, str, 0L));
+    int slen = strlen (str);
+    if (!data) {
+        takeUtf8 (g_strdup (str), slen);
+    } else {
+        if (data->buf_len < 0)
+            data->buf_len = strlen (data->ptr);
+        takeUtf8 (strCat (data->ptr, data->buf_len, str, slen),
+                  data->buf_len + slen);
+    }
     return *this;
 }
 
@@ -309,7 +344,7 @@ GLibString GLibString::mid (const int index, unsigned len) const {
     if (!eos || !*eos)
         str.takeUtf8 (g_strdup (off));
     else
-        str.takeUtf8 (g_strndup (off, eos-off));
+        str.takeUtf8 (g_strndup (off, eos-off), eos - off);
     return str;
 }
 
@@ -331,9 +366,10 @@ void GLibString::truncate (int pos) {
         if (pos == 0) {  // speed up clearing string
             clear ();
         } else {
-            gchar * epos = g_utf8_offset_to_pointer (data->ptr, pos);
+            gchar *epos = g_utf8_offset_to_pointer (data->ptr, pos);
             if (epos)
-                takeUtf8 (g_strndup (data->ptr, epos - data->ptr));
+                takeUtf8 (g_strndup (data->ptr, epos - data->ptr),
+                          epos - data->ptr);
         }
     }
 }
@@ -722,6 +758,7 @@ bool GnomeVFSGetJob::error () {
 #endif
 //-----------------------------------------------------------------------------
 
+#include <conic.h>
 #include <curl/curl.h>
 
 static gint easyCompare (gconstpointer a, gconstpointer b) {
@@ -735,9 +772,11 @@ namespace KMPlayer {
         ~CurlGetJob ();
 
         void init (IOJob *job);
+        void start ();
 
         static size_t headerLine( void *p, size_t sz, size_t n, void *strm);
         static size_t dataReceived (void *p, size_t sz, size_t n, void *strm);
+        static size_t writePost (void *p, size_t sz, size_t n, void *strm);
         static gboolean findKilledJob (gpointer key, gpointer val, gpointer d);
 
         IOJobListener * m_receiver;
@@ -747,6 +786,8 @@ namespace KMPlayer {
 
         String url;
         String content_type;
+        ByteArray post_data;
+        curl_slist *headers;
         CURL *easy;
         bool quiet;
         bool killed;
@@ -761,6 +802,7 @@ namespace {
     struct FdWatch {
         int fd;
         int chan_read_watch;
+        int chan_write_watch;
     };
 
 }
@@ -785,6 +827,9 @@ CurlGetJob::findKilledJob (gpointer key, gpointer value, gpointer data) {
 }
 
 static gboolean curlChannelRead (GIOChannel *src, GIOCondition c, gpointer d);
+static gboolean curlChannelWrite (GIOChannel *src, GIOCondition c, gpointer d);
+static gboolean curlTimeOut (void *);
+static void curlPerformMulti ();
 bool CurlGetJob::global_init = false;
 CURLM *CurlGetJob::multi = NULL;
 static GTree *easy_list = NULL;
@@ -793,6 +838,27 @@ static int fd_watch_array_len;
 static gint curl_timeout_timer;
 static bool in_perform_multi;
 static bool curl_job_killed;
+static ConIcConnection *con_ic_connection;
+#ifdef __ARMEL__
+static ConIcConnectionStatus ic_status = CON_IC_STATUS_DISCONNECTED;
+#else
+static ConIcConnectionStatus ic_status = CON_IC_STATUS_CONNECTED;
+#endif
+static GSList *ic_connect_waiters;
+static bool ic_connect_request;
+
+void icConnection (ConIcConnection *conn, ConIcConnectionEvent *ev, gpointer) {
+    ic_connect_request = false;
+    ic_status = con_ic_connection_event_get_status (ev);
+    if (CON_IC_STATUS_CONNECTED == ic_status && ic_connect_waiters) {
+        ic_connect_waiters = g_slist_reverse (ic_connect_waiters);
+        for (GSList *sl = ic_connect_waiters; sl; sl = sl->next)
+            ((CurlGetJob *)sl->data)->start ();
+        g_slist_free (ic_connect_waiters);
+        ic_connect_waiters = NULL;
+        curlPerformMulti ();
+    }
+}
 
 static void curlChannelDestroy (gpointer) {
     //debugLog() << "channel destoyed" << endl;
@@ -805,6 +871,7 @@ static void updateReadWatches() {
     FD_ZERO (&fd_excep);
     int maxfd = 0;
     curl_multi_fdset (CurlGetJob::multi, &fd_read, &fd_write, &fd_excep, &maxfd);
+    ++maxfd;
     if (maxfd > 0 && !fd_watch_array) {
         fd_watch_array = new FdWatch[maxfd];
         memset (fd_watch_array, 0, maxfd * sizeof (FdWatch));
@@ -819,16 +886,31 @@ static void updateReadWatches() {
     }
     for (int i = 3; i < fd_watch_array_len; i++) {
         if (FD_ISSET (i, &fd_read)) {
-            if (!fd_watch_array[i].fd) {
+            //debugLog () << "update watches for read fd=" << i << endl;
+            if (!fd_watch_array[i].chan_read_watch) {
                 fd_watch_array[i].fd = i;
                 GIOChannel *channel = g_io_channel_unix_new (i);
                 g_io_channel_set_encoding (channel, NULL, NULL);
                 fd_watch_array[i].chan_read_watch = g_io_add_watch_full (channel, G_PRIORITY_DEFAULT, G_IO_IN, curlChannelRead, NULL, curlChannelDestroy);
                 g_io_channel_unref (channel);
             }
-        } else if (fd_watch_array[i].fd) {
+        } else if (fd_watch_array[i].chan_read_watch) {
             g_source_remove (fd_watch_array[i].chan_read_watch);
-            fd_watch_array[i].fd = fd_watch_array[i].chan_read_watch = 0;
+            fd_watch_array[i].chan_read_watch = 0;
+        }
+        if (!fd_watch_array[i].chan_read_watch && FD_ISSET (i, &fd_write)) {
+            // connecting ..
+            //debugLog () << "update watches for write fd=" << i << endl;
+            if (!fd_watch_array[i].chan_write_watch) {
+                fd_watch_array[i].fd = i;
+                GIOChannel *channel = g_io_channel_unix_new (i);
+                g_io_channel_set_encoding (channel, NULL, NULL);
+                fd_watch_array[i].chan_write_watch = g_io_add_watch_full (channel, G_PRIORITY_DEFAULT, G_IO_OUT, curlChannelWrite, NULL, curlChannelDestroy);
+                g_io_channel_unref (channel);
+            }
+        } else if (fd_watch_array[i].chan_write_watch) {
+            g_source_remove (fd_watch_array[i].chan_write_watch);
+            fd_watch_array[i].chan_write_watch = 0;
         }
     }
 }
@@ -839,6 +921,7 @@ static void curlPerformMulti () {
 
     int running_handles = 0;
     CURLMcode curlCode;
+
     do {
         curl_job_killed = false;
         curlCode = curl_multi_perform (CurlGetJob::multi, &running_handles);
@@ -852,6 +935,7 @@ static void curlPerformMulti () {
                     break;
             } while (true);
     } while (CURLM_CALL_MULTI_PERFORM == curlCode && running_handles > 0);
+
     int messages;
     CURLMsg* msg = curl_multi_info_read (CurlGetJob::multi, &messages);
     while (msg) {
@@ -865,6 +949,9 @@ static void curlPerformMulti () {
     updateReadWatches();
 
     in_perform_multi = false;
+
+    if (!curl_timeout_timer && g_tree_height (easy_list))
+        curl_timeout_timer = g_timeout_add (250, curlTimeOut, NULL);
 }
 
 static gboolean curlTimeOut (void *) {
@@ -880,6 +967,10 @@ static gboolean curlChannelRead (GIOChannel *src, GIOCondition c, gpointer d) {
     curlPerformMulti ();
 }
 
+static gboolean curlChannelWrite (GIOChannel *src, GIOCondition c, gpointer d) {
+    //debugLog() << "curl connected" << endl;
+    curlPerformMulti ();
+}
 size_t CurlGetJob::headerLine( void *ptr, size_t size, size_t nmemb, void *stream) {
     IOJob *job = (IOJob *) stream;
     CurlGetJob *self = job->d;
@@ -926,11 +1017,29 @@ size_t CurlGetJob::dataReceived (void *ptr, size_t size, size_t nmemb, void *str
         self->m_receiver->jobData (job, data);
     return size * nmemb;
 }
+
+size_t CurlGetJob::writePost (void *p, size_t sz, size_t n, void *stream) {
+    IOJob *job = (IOJob *) stream;
+    CurlGetJob *self = job->d;
+    int plen = self->post_data.size ();
+    if (!plen)
+        return 0;
+    int count = plen;
+    if (count > sz * n)
+        count = sz * n;
+    memcpy (p, self->post_data.data (), count);
+    if (plen > count) {
+        memmove (self->post_data.data(), self->post_data.data() + count, plen - count);
+    }
+    self->post_data.resize (plen - count);
+    return count;
+}
+
 static int job_count;
 
 KMPLAYER_NO_CDTOR_EXPORT
 CurlGetJob::CurlGetJob (IOJobListener * rec, const String & u)
- : m_receiver (rec), url (u),
+ : m_receiver (rec), url (u), headers (NULL), easy (NULL),
    quiet (false), killed (false), redirected (false),
    error_code (0), content_length (0)
 {}
@@ -940,6 +1049,9 @@ void CurlGetJob::init (IOJob *job) {
         global_init = !curl_global_init (CURL_GLOBAL_ALL);
         easy_list = g_tree_new (easyCompare);
         debugLog() << "global curl init: " << global_init << endl;
+        con_ic_connection = con_ic_connection_new ();
+        g_signal_connect (G_OBJECT (con_ic_connection), "connection-event",
+                G_CALLBACK (icConnection), NULL);
     }
     if (!multi)
         multi = curl_multi_init ();
@@ -952,6 +1064,12 @@ void CurlGetJob::init (IOJob *job) {
     curl_easy_setopt (easy, CURLOPT_WRITEDATA, job);
     curl_easy_setopt (easy, CURLOPT_HEADERFUNCTION, CurlGetJob::headerLine);
     curl_easy_setopt (easy, CURLOPT_WRITEHEADER, job);
+    if (post_data.size ()) {
+        curl_easy_setopt (easy, CURLOPT_POST, 1);
+        curl_easy_setopt (easy, CURLOPT_POSTFIELDSIZE, post_data.size ());
+        curl_easy_setopt (easy, CURLOPT_READFUNCTION, CurlGetJob::writePost);
+        curl_easy_setopt (easy, CURLOPT_READDATA, job);
+    }
     curl_easy_setopt (easy, CURLOPT_FOLLOWLOCATION, 1);
     curl_easy_setopt (easy, CURLOPT_MAXREDIRS, 10);
     //CURLSH *curl_share_init( );
@@ -960,26 +1078,45 @@ void CurlGetJob::init (IOJob *job) {
     //curl_share_cleanup(CURLSH * share_handle );
     curl_easy_setopt (easy, CURLOPT_DNS_CACHE_TIMEOUT, 60 * 5);
     curl_easy_setopt (easy, CURLOPT_ENCODING, "");
-    curl_easy_setopt (easy, CURLOPT_USERAGENT, "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux armv6l; U) Opera 8.5 [en_US] Tablet browser 0.0.14 RX-34_2007SE_4.2007.26-8");
+    curl_easy_setopt (easy, CURLOPT_USERAGENT, "KMPlayer/" VERSION);
     curl_easy_setopt (easy, CURLOPT_SSL_VERIFYPEER, 0);
     curl_easy_setopt (easy, CURLOPT_URL, (const char *)url);
+    if (headers)
+        curl_easy_setopt (easy, CURLOPT_HTTPHEADER, headers);
     //if (!strncmp (u, "http", 4))
     //    curl_easy_setopt (easy, CURLOPT_HTTPGET, true);
+    if (CON_IC_STATUS_CONNECTED == ic_status) {
+        start ();
+    } else {
+        if (!ic_connect_request)
+            ic_connect_request = con_ic_connection_connect (con_ic_connection,
+                    CON_IC_CONNECT_FLAG_NONE);
+        if (ic_connect_request)
+            ic_connect_waiters = g_slist_prepend (ic_connect_waiters, this);
+        else
+            job->kill (false);
+    }
+}
+
+void CurlGetJob::start () {
     CURLMcode type = curl_multi_add_handle (multi, easy);
     if (type && type != CURLM_CALL_MULTI_PERFORM)
         debugLog() << "CurlGetJob add to multi failure" << endl;
-
-    if (!curl_timeout_timer)
-        curl_timeout_timer = g_timeout_add (50, curlTimeOut, NULL);
+    else if (!ic_connect_waiters)
+        curlPerformMulti ();
 }
 
 KMPLAYER_NO_CDTOR_EXPORT CurlGetJob::~CurlGetJob () {
     //debugLog() << "CurlGetJob::~CurlGetJob " << easy << " " << --job_count << endl;
+    if (ic_connect_waiters)
+        ic_connect_waiters = g_slist_remove (ic_connect_waiters, this);
     if (easy) {
         curl_multi_remove_handle (multi, easy);
         curl_easy_cleanup (easy);
         g_tree_remove (easy_list, easy);
     }
+    if (headers)
+        curl_slist_free_all (headers);
     if (0) {
         curl_multi_cleanup (multi);
         multi = NULL;
@@ -987,7 +1124,6 @@ KMPLAYER_NO_CDTOR_EXPORT CurlGetJob::~CurlGetJob () {
 }
 
 KMPLAYER_NO_EXPORT IOJob::IOJob (IOJobPrivate *priv) : d (priv) {
-    priv->init (this);
 }
 
 KMPLAYER_NO_EXPORT IOJob::~IOJob () {
@@ -999,6 +1135,18 @@ KMPLAYER_NO_EXPORT IOJob::~IOJob () {
     delete d;
 }
 
+KMPLAYER_NO_EXPORT void IOJob::setHeader (const String &header) {
+    d->headers = curl_slist_append (d->headers, (const char *) header);
+}
+
+KMPLAYER_NO_EXPORT void IOJob::setHttpPostData (const ByteArray &data) {
+    d->post_data = data;
+}
+
+KMPLAYER_NO_EXPORT void IOJob::start () {
+    d->init (this);
+}
+
 KMPLAYER_NO_EXPORT void IOJob::kill (bool q) {
     d->quiet = q;
     d->killed = true;
